@@ -122,12 +122,9 @@ public:
     void updateTexture(const QSize &size, const QString &elementId)
     {
         QQuickWindow::CreateTextureOptions options;
-//Qt < 5.3.2. has a crash on some atlas textures
-#if (QT_VERSION > QT_VERSION_CHECK(5, 3, 2))
         if (m_fitMode != Tile) {
             options = QQuickWindow::TextureCanUseAtlas;
         }
-#endif
         setTexture(s_cache->loadTexture(m_frameSvg->window(), m_frameSvg->frameSvg()->image(size, elementId), options));
     }
 
@@ -184,7 +181,6 @@ FrameSvgItemMargins::FrameSvgItemMargins(Plasma::FrameSvg *frameSvg, QObject *pa
       m_fixed(false)
 {
     //qDebug() << "margins at: " << left() << top() << right() << bottom();
-    connect(m_frameSvg, SIGNAL(repaintNeeded()), this, SLOT(update()));
 }
 
 qreal FrameSvgItemMargins::left() const
@@ -233,6 +229,13 @@ qreal FrameSvgItemMargins::vertical() const
     return top() + bottom();
 }
 
+QVector<qreal> FrameSvgItemMargins::margins() const
+{
+    qreal left, top, right, bottom;
+    m_frameSvg->getMargins(left, top, right, bottom);
+    return {left, top, right, bottom};
+}
+
 void FrameSvgItemMargins::update()
 {
     emit marginsChanged();
@@ -255,33 +258,52 @@ bool FrameSvgItemMargins::isFixed() const
 
 FrameSvgItem::FrameSvgItem(QQuickItem *parent)
     : QQuickItem(parent),
+      m_margins(nullptr),
+      m_fixedMargins(nullptr),
       m_textureChanged(false),
       m_sizeChanged(false),
       m_fastPath(true)
 {
     m_frameSvg = new Plasma::FrameSvg(this);
-    m_margins = new FrameSvgItemMargins(m_frameSvg, this);
-    m_fixedMargins = new FrameSvgItemMargins(m_frameSvg, this);
-    m_fixedMargins->setFixed(true);
     setFlag(ItemHasContents, true);
-    connect(m_frameSvg, SIGNAL(repaintNeeded()), this, SLOT(doUpdate()));
-    connect(&m_units, &Units::devicePixelRatioChanged, this, &FrameSvgItem::updateDevicePixelRatio);
+    connect(m_frameSvg, &FrameSvg::repaintNeeded, this, &FrameSvgItem::doUpdate);
+    connect(&Units::instance(), &Units::devicePixelRatioChanged, this, &FrameSvgItem::updateDevicePixelRatio);
     connect(m_frameSvg, &Svg::fromCurrentThemeChanged, this, &FrameSvgItem::fromCurrentThemeChanged);
+    connect(m_frameSvg, &Svg::statusChanged, this, &FrameSvgItem::statusChanged);
 }
 
 FrameSvgItem::~FrameSvgItem()
 {
 }
 
+class CheckMarginsChange
+{
+public:
+    CheckMarginsChange(FrameSvgItemMargins *margins)
+        : m_oldMargins(margins ? margins->margins() : QVector<qreal>()), m_margins(margins)
+    {}
+
+    ~CheckMarginsChange() {
+        if (m_margins && m_margins->margins() != m_oldMargins) {
+            m_margins->update();
+        }
+    }
+
+    const QVector<qreal> m_oldMargins;
+    FrameSvgItemMargins *const m_margins;
+};
+
 void FrameSvgItem::setImagePath(const QString &path)
 {
     if (m_frameSvg->imagePath() == path) {
         return;
     }
 
+    CheckMarginsChange checkMargins(m_margins);
+    CheckMarginsChange checkFixedMargins(m_fixedMargins);
+
     updateDevicePixelRatio();
     m_frameSvg->setImagePath(path);
-    m_frameSvg->setElementPrefix(m_prefix);
 
     if (implicitWidth() <= 0) {
         setImplicitWidth(m_frameSvg->marginSize(Plasma::Types::LeftMargin) + m_frameSvg->marginSize(Plasma::Types::RightMargin));
@@ -292,10 +314,10 @@ void FrameSvgItem::setImagePath(const QString &path)
     }
 
     emit imagePathChanged();
-    m_margins->update();
-    m_fixedMargins->update();
 
     if (isComponentComplete()) {
+        applyPrefixes();
+
         m_frameSvg->resizeFrame(QSizeF(width(), height()));
         m_textureChanged = true;
         update();
@@ -307,14 +329,25 @@ QString FrameSvgItem::imagePath() const
     return m_frameSvg->imagePath();
 }
 
-void FrameSvgItem::setPrefix(const QString &prefix)
+void FrameSvgItem::setPrefix(const QVariant &prefixes)
 {
-    if (m_prefix == prefix) {
+    QStringList prefixList;
+    //is this a simple string?
+    if (prefixes.canConvert<QString>()) {
+        prefixList << prefixes.toString();
+    } else if (prefixes.canConvert<QStringList>()) {
+        prefixList = prefixes.toStringList();
+    }
+
+    if (m_prefixes == prefixList) {
         return;
     }
 
-    m_frameSvg->setElementPrefix(prefix);
-    m_prefix = prefix;
+    CheckMarginsChange checkMargins(m_margins);
+    CheckMarginsChange checkFixedMargins(m_fixedMargins);
+
+    m_prefixes = prefixList;
+    applyPrefixes();
 
     if (implicitWidth() <= 0) {
         setImplicitWidth(m_frameSvg->marginSize(Plasma::Types::LeftMargin) + m_frameSvg->marginSize(Plasma::Types::RightMargin));
@@ -325,8 +358,6 @@ void FrameSvgItem::setPrefix(const QString &prefix)
     }
 
     emit prefixChanged();
-    m_margins->update();
-    m_fixedMargins->update();
 
     if (isComponentComplete()) {
         m_frameSvg->resizeFrame(QSizeF(width(), height()));
@@ -335,18 +366,30 @@ void FrameSvgItem::setPrefix(const QString &prefix)
     }
 }
 
-QString FrameSvgItem::prefix() const
+QVariant FrameSvgItem::prefix() const
 {
-    return m_prefix;
+    return m_prefixes;
 }
 
-FrameSvgItemMargins *FrameSvgItem::margins() const
+QString FrameSvgItem::usedPrefix() const
 {
+    return m_frameSvg->prefix();
+}
+
+FrameSvgItemMargins *FrameSvgItem::margins()
+{
+    if (!m_margins) {
+        m_margins = new FrameSvgItemMargins(m_frameSvg, this);
+    }
     return m_margins;
 }
 
-FrameSvgItemMargins *FrameSvgItem::fixedMargins() const
+FrameSvgItemMargins *FrameSvgItem::fixedMargins()
 {
+    if (!m_fixedMargins) {
+        m_fixedMargins = new FrameSvgItemMargins(m_frameSvg, this);
+        m_fixedMargins->setFixed(true);
+    }
     return m_fixedMargins;
 }
 
@@ -371,16 +414,27 @@ bool FrameSvgItem::fromCurrentTheme() const
     return m_frameSvg->fromCurrentTheme();
 }
 
+void FrameSvgItem::setStatus(Plasma::Svg::Status status)
+{
+    m_frameSvg->setStatus(status);
+}
+
+Plasma::Svg::Status FrameSvgItem::status() const
+{
+    return m_frameSvg->status();
+}
+
 void FrameSvgItem::setEnabledBorders(const Plasma::FrameSvg::EnabledBorders borders)
 {
     if (m_frameSvg->enabledBorders() == borders) {
         return;
     }
 
+    CheckMarginsChange checkMargins(m_margins);
+
     m_frameSvg->setEnabledBorders(borders);
     emit enabledBordersChanged();
     m_textureChanged = true;
-    m_margins->update();
     update();
 }
 
@@ -406,6 +460,16 @@ void FrameSvgItem::geometryChanged(const QRectF &newGeometry,
 
 void FrameSvgItem::doUpdate()
 {
+    if (m_frameSvg->isRepaintBlocked()) {
+        return;
+    }
+
+    CheckMarginsChange checkMargins(m_margins);
+    CheckMarginsChange checkFixedMargins(m_fixedMargins);
+
+    //if the theme changed, the available prefix may have changed as well
+    applyPrefixes();
+
     if (implicitWidth() <= 0) {
         setImplicitWidth(m_frameSvg->marginSize(Plasma::Types::LeftMargin) + m_frameSvg->marginSize(Plasma::Types::RightMargin));
     }
@@ -419,11 +483,18 @@ void FrameSvgItem::doUpdate()
     bool hasComposeOverBorder = m_frameSvg->hasElement(prefix % QLatin1String("hint-compose-over-border")) &&
                 m_frameSvg->hasElement(QLatin1String("mask-") % prefix % QLatin1String("center"));
     m_fastPath = !hasOverlay && !hasComposeOverBorder;
+
+    //software rendering (at time of writing Qt5.10) doesn't seem to like our tiling/stretching in the 9-tiles.
+    //also when using QPainter it's arguably faster to create and cache pixmaps of the whole frame, which is what the slow path does
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 9, 0))
+    if (QQuickWindow::sceneGraphBackend() == QLatin1String("software")) {
+        m_fastPath = false;
+    }
+#endif
     m_textureChanged = true;
 
     update();
-    m_margins->update();
-    m_fixedMargins->update();
+
     emit repaintNeeded();
 }
 
@@ -435,9 +506,9 @@ Plasma::FrameSvg *FrameSvgItem::frameSvg() const
 QSGNode *FrameSvgItem::updatePaintNode(QSGNode *oldNode, QQuickItem::UpdatePaintNodeData *)
 {
     if (!window() || !m_frameSvg ||
-        (!m_frameSvg->hasElementPrefix(m_frameSvg->actualPrefix()) && !m_frameSvg->hasElementPrefix(m_prefix))) {
+        (!m_frameSvg->hasElementPrefix(m_frameSvg->actualPrefix()) && !m_frameSvg->hasElementPrefix(m_frameSvg->prefix()))) {
         delete oldNode;
-        return Q_NULLPTR;
+        return nullptr;
     }
 
     if (m_fastPath) {
@@ -458,14 +529,30 @@ QSGNode *FrameSvgItem::updatePaintNode(QSGNode *oldNode, QQuickItem::UpdatePaint
             FrameItemNode::FitMode centerFitMode = tileCenter ? FrameItemNode::Tile: FrameItemNode::Stretch;
 
             new FrameItemNode(this, FrameSvg::NoBorder, centerFitMode, oldNode);
-            new FrameItemNode(this, FrameSvg::TopBorder | FrameSvg::LeftBorder, FrameItemNode::FastStretch, oldNode);
-            new FrameItemNode(this, FrameSvg::TopBorder | FrameSvg::RightBorder, FrameItemNode::FastStretch, oldNode);
-            new FrameItemNode(this, FrameSvg::TopBorder, borderFitMode, oldNode);
-            new FrameItemNode(this, FrameSvg::BottomBorder, borderFitMode, oldNode);
-            new FrameItemNode(this, FrameSvg::BottomBorder | FrameSvg::LeftBorder, FrameItemNode::FastStretch, oldNode);
-            new FrameItemNode(this, FrameSvg::BottomBorder | FrameSvg::RightBorder, FrameItemNode::FastStretch, oldNode);
-            new FrameItemNode(this, FrameSvg::LeftBorder,  borderFitMode, oldNode);
-            new FrameItemNode(this, FrameSvg::RightBorder, borderFitMode, oldNode);
+            if (enabledBorders() & (FrameSvg::TopBorder | FrameSvg::LeftBorder)) {
+                new FrameItemNode(this, FrameSvg::TopBorder | FrameSvg::LeftBorder, FrameItemNode::FastStretch, oldNode);
+            }
+            if (enabledBorders() & (FrameSvg::TopBorder | FrameSvg::RightBorder)) {
+                new FrameItemNode(this, FrameSvg::TopBorder | FrameSvg::RightBorder, FrameItemNode::FastStretch, oldNode);
+            }
+            if (enabledBorders() & FrameSvg::TopBorder) {
+                new FrameItemNode(this, FrameSvg::TopBorder, borderFitMode, oldNode);
+            }
+            if (enabledBorders() & FrameSvg::BottomBorder) {
+                new FrameItemNode(this, FrameSvg::BottomBorder, borderFitMode, oldNode);
+            }
+            if (enabledBorders() & (FrameSvg::BottomBorder | FrameSvg::LeftBorder)) {
+                new FrameItemNode(this, FrameSvg::BottomBorder | FrameSvg::LeftBorder, FrameItemNode::FastStretch, oldNode);
+            }
+            if (enabledBorders() & (FrameSvg::BottomBorder | FrameSvg::RightBorder)) {
+                new FrameItemNode(this, FrameSvg::BottomBorder | FrameSvg::RightBorder, FrameItemNode::FastStretch, oldNode);
+            }
+            if (enabledBorders() & FrameSvg::LeftBorder) {
+                new FrameItemNode(this, FrameSvg::LeftBorder,  borderFitMode, oldNode);
+            }
+            if (enabledBorders() & FrameSvg::RightBorder) {
+                new FrameItemNode(this, FrameSvg::RightBorder, borderFitMode, oldNode);
+            }
 
             m_sizeChanged = true;
             m_textureChanged = false;
@@ -475,9 +562,10 @@ QSGNode *FrameSvgItem::updatePaintNode(QSGNode *oldNode, QQuickItem::UpdatePaint
             FrameNode* frameNode = static_cast<FrameNode*>(oldNode);
             QSize frameSize(width(), height());
             QRect geometry = frameNode->contentsRect(frameSize);
-            for(int i = 0; i<oldNode->childCount(); ++i) {
-                FrameItemNode* it = static_cast<FrameItemNode*>(oldNode->childAtIndex(i));
-                it->reposition(geometry, frameSize);
+            QSGNode *node = oldNode->firstChild();
+            while (node) {
+                static_cast<FrameItemNode *>(node)->reposition(geometry, frameSize);
+                node = node->nextSibling();
             }
 
             m_sizeChanged = false;
@@ -505,10 +593,20 @@ QSGNode *FrameSvgItem::updatePaintNode(QSGNode *oldNode, QQuickItem::UpdatePaint
     return oldNode;
 }
 
+void FrameSvgItem::classBegin()
+{
+    QQuickItem::classBegin();
+    m_frameSvg->setRepaintBlocked(true);
+}
+
 void FrameSvgItem::componentComplete()
 {
+    CheckMarginsChange checkMargins(m_margins);
+    CheckMarginsChange checkFixedMargins(m_fixedMargins);
+
     QQuickItem::componentComplete();
     m_frameSvg->resizeFrame(QSize(width(), height()));
+    m_frameSvg->setRepaintBlocked(false);
     m_textureChanged = true;
 }
 
@@ -521,9 +619,43 @@ void FrameSvgItem::updateDevicePixelRatio()
     } else {
         m_frameSvg->setDevicePixelRatio(qMax<qreal>(1.0, floor(qApp->devicePixelRatio())));
     }
-    m_frameSvg->setScaleFactor(qMax<qreal>(1.0, floor(m_units.devicePixelRatio())));
+    m_frameSvg->setScaleFactor(qMax<qreal>(1.0, floor(Units::instance().devicePixelRatio())));
     m_textureChanged = true;
 }
 
+void FrameSvgItem::applyPrefixes()
+{
+
+    if (m_frameSvg->imagePath().isEmpty()) {
+        return;
+    }
+
+    const QString oldPrefix = m_frameSvg->prefix();
+
+    if (m_prefixes.isEmpty()) {
+        m_frameSvg->setElementPrefix(QString());
+        if (oldPrefix != m_frameSvg->prefix()) {
+            emit usedPrefixChanged();
+        }
+        return;
+    }
+
+    bool found = false;
+    for (const QString &prefix : qAsConst(m_prefixes)) {
+        if (m_frameSvg->hasElementPrefix(prefix)) {
+            m_frameSvg->setElementPrefix(prefix);
+            found = true;
+            break;
+        }
+    }
+    if (!found) {
+        //this setElementPrefix is done to keep the same behavior as before, when it was a simple string
+        m_frameSvg->setElementPrefix(m_prefixes.constLast());
+    }
+    if (oldPrefix != m_frameSvg->prefix()) {
+        emit usedPrefixChanged();
+    }
+}
+
 } // Plasma namespace
 
