@@ -33,7 +33,6 @@
 
 #include <kplugininfo.h>
 #include <kservice.h>
-#include <kservicetypetrader.h>
 #include <klocalizedstring.h>
 
 #include "datacontainer.h"
@@ -65,7 +64,7 @@ DataEngine::DataEngine(const KPluginInfo &plugin, QObject *parent)
 
 DataEngine::DataEngine(QObject *parent, const QVariantList &args)
     : QObject(parent),
-      d(new DataEnginePrivate(this, KPluginInfo(args)))
+      d(new DataEnginePrivate(this, KPluginInfo(args), args))
 {
     if (d->script) {
         d->setupScriptSupport();
@@ -75,7 +74,7 @@ DataEngine::DataEngine(QObject *parent, const QVariantList &args)
 
 DataEngine::~DataEngine()
 {
-    //qDebug() << objectName() << ": bye bye birdy! ";
+    //qCDebug(LOG_PLASMA) << objectName() << ": bye bye birdy! ";
     delete d;
 }
 
@@ -109,7 +108,7 @@ void DataEngine::connectSource(const QString &source, QObject *visualization,
                                uint pollingInterval,
                                Plasma::Types::IntervalAlignment intervalAlignment) const
 {
-    //qDebug() << "connectSource" << source;
+    //qCDebug(LOG_PLASMA) << "connectSource" << source;
     bool newSource;
     DataContainer *s = d->requestSource(source, &newSource);
 
@@ -123,7 +122,7 @@ void DataEngine::connectSource(const QString &source, QObject *visualization,
         }
         d->connectSource(s, visualization, pollingInterval, intervalAlignment,
                          !newSource || pollingInterval > 0);
-        //qDebug() << " ==> source connected";
+        //qCDebug(LOG_PLASMA) << " ==> source connected";
     }
 }
 
@@ -163,7 +162,7 @@ bool DataEngine::updateSourceEvent(const QString &source)
     if (d->script) {
         return d->script->updateSourceEvent(source);
     } else {
-        //qDebug() << source;
+        //qCDebug(LOG_PLASMA) << source;
         return false; //TODO: should this be true to trigger, even needless, updates on every tick?
     }
 }
@@ -234,9 +233,9 @@ void DataEngine::removeData(const QString &source, const QString &key)
 void DataEngine::setModel(const QString &source, QAbstractItemModel *model)
 {
     if (model) {
-        setData(source, "HasModel", true);
+        setData(source, QStringLiteral("HasModel"), true);
     } else {
-        removeData(source, "HasModel");
+        removeData(source, QStringLiteral("HasModel"));
     }
 
     Plasma::DataContainer *s = containerForSource(source);
@@ -261,7 +260,7 @@ void DataEngine::addSource(DataContainer *source)
 {
     if (d->sources.contains(source->objectName())) {
 #ifndef NDEBUG
-        // qDebug() << "source named \"" << source->objectName() << "\" already exists.";
+        // qCDebug(LOG_PLASMA) << "source named \"" << source->objectName() << "\" already exists.";
 #endif
         return;
     }
@@ -300,9 +299,9 @@ void DataEngine::removeSource(const QString &source)
     if (it != d->sources.end()) {
         DataContainer *s = it.value();
         s->d->store();
+        d->sources.erase(it);
         s->disconnect(this);
         s->deleteLater();
-        d->sources.erase(it);
         emit sourceRemoved(source);
     }
 }
@@ -343,21 +342,21 @@ QHash<QString, DataContainer *> DataEngine::containerDict() const
 
 void DataEngine::timerEvent(QTimerEvent *event)
 {
-    //qDebug();
+    //qCDebug(LOG_PLASMA);
     if (event->timerId() == d->updateTimerId) {
         // if the freq update is less than 0, don't bother
         if (d->minPollingInterval < 0) {
-            //qDebug() << "uh oh.. no polling allowed!";
+            //qCDebug(LOG_PLASMA) << "uh oh.. no polling allowed!";
             return;
         }
 
         // minPollingInterval
-        if (d->updateTimestamp.elapsed() < d->minPollingInterval) {
-            //qDebug() << "hey now.. slow down!";
+        if (d->updateTimer.elapsed() < d->minPollingInterval) {
+            //qCDebug(LOG_PLASMA) << "hey now.. slow down!";
             return;
         }
 
-        d->updateTimestamp.restart();
+        d->updateTimer.start();
         updateAllSources();
     } else if (event->timerId() == d->checkSourcesTimerId) {
         killTimer(d->checkSourcesTimerId);
@@ -378,8 +377,10 @@ void DataEngine::updateAllSources()
     QHashIterator<QString, Plasma::DataContainer *> it(d->sources);
     while (it.hasNext()) {
         it.next();
-        //qDebug() << "updating" << it.key();
-        updateSourceEvent(it.key());
+        //qCDebug(LOG_PLASMA) << "updating" << it.key();
+        if (it.value()->isUsed()) {
+            updateSourceEvent(it.key());
+        }
     }
 
     d->scheduleSourcesUpdated();
@@ -388,7 +389,9 @@ void DataEngine::updateAllSources()
 void DataEngine::forceImmediateUpdateOfAllVisualizations()
 {
     foreach (DataContainer *source, d->sources) {
-        source->forceImmediateUpdate();
+        if (source->isUsed()) {
+            source->forceImmediateUpdate();
+        }
     }
 }
 
@@ -406,43 +409,42 @@ void DataEngine::setStorageEnabled(const QString &source, bool store)
 }
 
 // Private class implementations
-DataEnginePrivate::DataEnginePrivate(DataEngine *e, const KPluginInfo &info)
+DataEnginePrivate::DataEnginePrivate(DataEngine *e, const KPluginInfo &info, const QVariantList &args)
     : q(e),
       dataEngineDescription(info),
       refCount(-1), // first ref
       checkSourcesTimerId(0),
       updateTimerId(0),
       minPollingInterval(-1),
-      valid(true),
+      valid(false),
       script(0),
       package(0)
 {
-    updateTimestamp.start();
+    updateTimer.start();
 
     if (dataEngineDescription.isValid()) {
+        valid = true;
         e->setObjectName(dataEngineDescription.name());
-    } else {
-        e->setObjectName("NullEngine");
     }
 
     if (dataEngineDescription.isValid()) {
-        QString api = dataEngineDescription.property("X-Plasma-API").toString();
+        QString api = dataEngineDescription.property(QStringLiteral("X-Plasma-API")).toString();
 
         if (!api.isEmpty()) {
             const QString path =
                 QStandardPaths::locate(QStandardPaths::GenericDataLocation,
-                                       PLASMA_RELATIVE_DATA_INSTALL_DIR "/dataengines/" + dataEngineDescription.pluginName() + '/',
+                                       QStringLiteral(PLASMA_RELATIVE_DATA_INSTALL_DIR "/dataengines/") + dataEngineDescription.pluginName(),
                                        QStandardPaths::LocateDirectory);
-            package = new Package(PluginLoader::self()->loadPackage("Plasma/DataEngine", api));
+            package = new Package(PluginLoader::self()->loadPackage(QStringLiteral("Plasma/DataEngine"), api));
             package->setPath(path);
 
             if (package->isValid()) {
-                script = Plasma::loadScriptEngine(api, q);
+                script = Plasma::loadScriptEngine(api, q, args);
             }
 
             if (!script) {
 #ifndef NDEBUG
-                // qDebug() << "Could not create a" << api << "ScriptEngine for the"
+                // qCDebug(LOG_PLASMA) << "Could not create a" << api << "ScriptEngine for the"
                 //        << dataEngineDescription.name() << "DataEngine.";
 #endif
                 delete package;
@@ -465,7 +467,7 @@ void DataEnginePrivate::internalUpdateSource(DataContainer *source)
     if (minPollingInterval > 0 &&
             source->timeSinceLastUpdate() < (uint)minPollingInterval) {
         // skip updating this source; it's been too soon
-        //qDebug() << "internal update source is delaying" << source->timeSinceLastUpdate() << minPollingInterval;
+        //qCDebug(LOG_PLASMA) << "internal update source is delaying" << source->timeSinceLastUpdate() << minPollingInterval;
         //but fake an update so that the signalrelay that triggered this gets the data from the
         //recent update. this way we don't have to worry about queuing - the relay will send a
         //signal immediately and everyone else is undisturbed.
@@ -474,11 +476,11 @@ void DataEnginePrivate::internalUpdateSource(DataContainer *source)
     }
 
     if (q->updateSourceEvent(source->objectName())) {
-        //qDebug() << "queuing an update";
+        //qCDebug(LOG_PLASMA) << "queuing an update";
         scheduleSourcesUpdated();
     }/* else {
 #ifndef NDEBUG
-        // qDebug() << "no update";
+        // qCDebug(LOG_PLASMA) << "no update";
 #endif
     }*/
 }
@@ -510,7 +512,7 @@ DataContainer *DataEnginePrivate::source(const QString &sourceName, bool createW
         return 0;
     }
 
-    //qDebug() << "DataEngine " << q->objectName() << ": could not find DataContainer " << sourceName << ", creating";
+    //qCDebug(LOG_PLASMA) << "DataEngine " << q->objectName() << ": could not find DataContainer " << sourceName << ", creating";
     DataContainer *s = new DataContainer(q);
     s->setObjectName(sourceName);
     sources.insert(sourceName, s);
@@ -526,7 +528,7 @@ void DataEnginePrivate::connectSource(DataContainer *s, QObject *visualization,
                                       Plasma::Types::IntervalAlignment align,
                                       bool immediateCall)
 {
-    //qDebug() << "connect source called" << s->objectName() << "with interval" << pollingInterval;
+    //qCDebug(LOG_PLASMA) << "connect source called" << s->objectName() << "with interval" << pollingInterval;
 
     if (pollingInterval > 0) {
         // never more frequently than allowed, never more than 20 times per second
@@ -540,7 +542,7 @@ void DataEnginePrivate::connectSource(DataContainer *s, QObject *visualization,
     if (immediateCall) {
         // we don't want to do an immediate call if we are simply
         // reconnecting
-        //qDebug() << "immediate call requested, we have:" << s->visualizationIsConnected(visualization);
+        //qCDebug(LOG_PLASMA) << "immediate call requested, we have:" << s->visualizationIsConnected(visualization);
         immediateCall = !s->data().isEmpty() &&
                         !s->visualizationIsConnected(visualization);
     }
@@ -554,7 +556,7 @@ void DataEnginePrivate::connectSource(DataContainer *s, QObject *visualization,
         if (s->d->model) {
             QMetaObject::invokeMethod(visualization, "modelChanged",
                                       Q_ARG(QString, s->objectName()),
-                                      Q_ARG(QAbstractItemModel *, s->d->model.data()));
+                                      Q_ARG(QAbstractItemModel*, s->d->model.data()));
         }
         s->d->dirty = false;
     }
@@ -579,12 +581,12 @@ DataContainer *DataEnginePrivate::requestSource(const QString &sourceName, bool
         *newSource = false;
     }
 
-    //qDebug() << "requesting source " << sourceName;
+    //qCDebug(LOG_PLASMA) << "requesting source " << sourceName;
     DataContainer *s = source(sourceName, false);
 
     if (!s) {
         // we didn't find a data source, so give the engine an opportunity to make one
-        /*// qDebug() << "DataEngine " << q->objectName()
+        /*// qCDebug(LOG_PLASMA) << "DataEngine " << q->objectName()
             << ": could not find DataContainer " << sourceName
             << " will create on request" << endl;*/
         waitingSourceRequest = sourceName;
@@ -616,7 +618,7 @@ void DataEnginePrivate::setupScriptSupport()
 
     /*
     #ifndef NDEBUG
-    // qDebug() << "sletting up script support, package is in" << package->path()
+    // qCDebug(LOG_PLASMA) << "sletting up script support, package is in" << package->path()
     #endif
              << "which is a" << package->structure()->type() << "package"
              << ", main script is" << package->filePath("mainscript");
